/*************************************************************************
*
*   Used with ICCARM and AARM.
*
*    (c) Copyright IAR Systems 2008
*
*    File name   : main.c
*    Description :
*
*   This example project shows how to use the IAR Embedded Workbench for ARM
*  to develop code for a Embedded Artsists LPC313x evaluation board. It shows
*  basic use of I/O, Timers, ADC, Interrup controller, SPI and SDRAM. The projec
*  also shows how the system can boot from the SPI NOR Flash.
*  It starts by blinking CPIO2_BOOT2 LED. The blinking period can be changed
*  by the trimer
*
*COMPATIBILITY
*=============
*
*   The Getting Started example project is compatible with Embedded Artsists
*  LPC313x evaluation board. By default the project is configured to use the
*  J-Link JTAG interface.
*
*CONFIGURATION
*=============
*
*  The Project contains the following configurations:
*
*  Debug_iRAM: This configuration is intended for debugging in the
*              microcontroller's internal RAM.
*
*  Debud_SDRAM: This configuration is intended for debugging in the external
*               SDRAM.
*
*  Debug_SPINOR_Boot_iRAM: This confuguration will download the example in the
*                          SPI NOR flash. The application is linked to run in
*                          the internal RAM and is loaded by LPC IROM
*                          bootloader. The configuration uses
*                          FlashEmbArtLPC313x_boot.xml flash loader.
*                          Note: This configuration uses a Hardware reset with
*                          delay to start IROM bootloader and allow it to copy
*                          the application. The boot jumpers should be
*                          configured for SPI NOR flash boot.
*
* Debug_SPINOR_Boot_SDRAM: This confuguration will download the example in the
*                          SPI NOR flash. The application is linked to run in
*                          the external SDRAM and a second level bootloader is
*                          linked to run in the internal RAM. The IROM
*                          bootloader copies the secon level bootloader from
*                          the SPI NOR to the interan RAM and starts it.
*                          The second level bootloader copies the rest of the
*                          application to SDRAM. The configuration uses
*                          FlashEmbArtLPC313x_boot.xml flash loader to program
*                          the secon level bootloader and
*                          FlashEmbArtLPC313x_img.xml flash loader to program
*                          the application.
*
*                          Note: This configuration uses a Hardware reset with
*                          delay to start IROM bootloader and allow it to copy
*                          the second level bootloader. The boot jumpers should
*                          be configured for SPI NOR flash boot.
*    History :
*    1. Date        : 18.3.2009
*       Author      : Stoyan Choynev
*       Description : initial revision.
*
*    $Revision: 30870 $
**************************************************************************/

/** include files **/
#include <NXP/iolpc3131.h>
#include <stdio.h>
#include <string.h>
#include "arm926ej_cp15_drv.h"
#include "arm_comm.h"
#include "drv_cgu.h"
#include "drv_spi.h"
#include "drv_spinor.h"
#include "drv_intc.h"
#include "drv_nand.h"
#include "uart.h"
#include "math.h"
/** local definitions **/
#define ISROM_MMU_TTBL  0x1201C000
#define SDRAM_BASE_ADDR 0x30000000
#define SDRAM_SIZE      0x02000000
#define IRAM_BASE_ADDR  0x11028000

#define LED1_SET  {IOCONF_GPIO_M1_SET = (1<<11);IOCONF_GPIO_M0_SET  = (1<<11);}
#define LED1_CLR  {IOCONF_GPIO_M1_SET = (1<<11);IOCONF_GPIO_M0_CLR  = (1<<11);}
#define LED2_SET  {IOCONF_GPIO_M1_SET = (1<<12);IOCONF_GPIO_M0_SET  = (1<<12);}
#define LED2_CLR  {IOCONF_GPIO_M1_SET = (1<<12);IOCONF_GPIO_M0_CLR  = (1<<12);}


#define   TIMER1_IN_FREQ       6MHZ
#define   TIMER1_TICK_MAX      1000   //in miliseconds
#define   TIMER1_TICK_MIN      20     //in miliseconds

#define   TMR1TICK(tick)        ((TIMER1_IN_FREQ/1000)*tick)/256 //Convert miliseconds in Tmr ticks

/** external functions **/
extern void InitSDRAM(void);
extern void Dly_us(Int32U Dly);

/** external data **/
Int32U Tmr1Tick = TMR1TICK(100);
/** internal functions **/
static int NAND_Test(void);
/*************************************************************************
* Function Name: Timer1IntrHandler
* Parameters: None
*
* Return: None
*
* Description: Timer 1 Interrupt service routine. Reload Timer
*              with Tmr1Tick and toggle the GPIO2_BOOT LED
*
*************************************************************************/
void Timer1IntrHandler (void)
{

	Timer1Load = Tmr1Tick;

	Timer1Clear = 0;              // clear timer interrupt

	if(IOCONF_GPIO_M0 & (1<<11))
	{
		LED1_CLR;
	}
	else
	{
		LED1_SET;
	}
}

/** public data **/

/** private data **/

/** public functions **/
/*************************************************************************
* Function Name: main
* Parameters: None
*
* Return: None
*
* Description: Getting Started main loop
*
*************************************************************************/
void main(void)
{
	UartLineCoding_t UartLineCoding;

	CP15_Mmu(FALSE);            // Disable MMU
	CP15_ICache(TRUE);          // Enable ICache

#ifndef DBG_SDRAM
	/*Init Clocks only if debug is not in SDRAM.
	if DBG_SDRAM the Clock is init by bootloader
	or mac file*/
	InitClock();
	/*Init SDRAM only if debug is not in SDRAM.
	if DBG_SDRAM the SDRAM is init by bootloader
	or mac file*/
	InitSDRAM();
#endif /*DBG_SDRAM*/

	/*Init SPI Nor Flash*/
	InitSPINOR();
	/*Init Interrupt Controller.
	Arm Vector Copy to beginnint of the IRAM*/
	INTC_Init((Int32U *)IRAM_BASE_ADDR);
	/*Remap IRAM at address 0*/
	SYSCREG_ARM926_901616_LP_SHADOW_POINT = IRAM_BASE_ADDR;
	/*Enable Timer 1 Clock*/
	CGU_Run_Clock(TIMER1_PCLK);
	/*Init Timer 1*/
	Timer1Ctrl_bit.Enable = 0;    // disable counting
	Timer1Ctrl_bit.Mode = 1;      // reload mode
	Timer1Ctrl_bit.PreScale = 2;  // prescaler = CGU clock rate / 256
	Timer1Load = Tmr1Tick;
	// set timer 0 period
	Timer1Clear = 0;              // clear timer pending interrupt
	/*Install Interrupt Service Routine,
	Priority 3*/
	INTC_IRQInstall(Timer1IntrHandler, TIMER0_INTCT1,
	3,0);
	/*Enable Timer 1 interrupt*/
	INTC_IntEnable(TIMER0_INTCT1, 1);

	// Init UART 0
	UartInit(4);

	// Update the baud rate
	UartLineCoding.dwDTERate = 115200;
	// Update the stop bits number
	UartLineCoding.bStopBitsFormat = UART_ONE_STOP_BIT;
	// Update the parity type
	UartLineCoding.bParityType = UART_NO_PARITY;
	// Update the word width
	UartLineCoding.bDataBits = 8-5;
	// Set UART line coding
	UartSetLineCoding(UartLineCoding);

	__enable_irq();

	/*Start Timer 1*/
	Timer1Ctrl_bit.Enable = 1;

	LED1_SET;

	NAND_Test();

	LED2_SET;

	while(1);
}
/** private functions **/

/*************************************************************************
* Function Name: NAND_Test
* Parameters: None
*
* Return: 0 on success, non 0 on error
*
* Description:
*
*************************************************************************/
static int NAND_Test(void)
{
	Int32S Block;
	int tries = 0;
	unsigned char pBuffer[516];
	char s[64];

	// wait for the user to press a key
	sprintf(s,"\n\rPress a key to start NAND test...");
	UartWrite((unsigned char *)s,strlen(s));
	while(UartRead((unsigned char *)s, 1) == 0)
		;
	
	sprintf(s,"\n\rStart NAND Test\n\r");
	UartWrite((unsigned char *)s,strlen(s));

	InitNAND();

    // display ID value
	sprintf(s,"\r\nNAND ID: 0x%08x\r\n",NandGetID());
	UartWrite((unsigned char *)s,strlen(s));

	for(Block = 0; Block < NAND_BLK_NUM; Block++)
	{
		// if there are too many errors terminate the test
		if(tries > 10) {
			break;
		}
		
		sprintf(s,"\r\nNAND Test Block: %u\r\n",Block);
		UartWrite((unsigned char *)s,strlen(s));

		if( FLASH_ERROR == NandCheckBlock(pBuffer,Block))
		{ // Do not erase invalid Block
			sprintf(s,"NAND Bad Block: %u\r\n",Block);
			UartWrite((unsigned char *)s,strlen(s));
			tries++;
		}
		else
		{
			if(FLASH_ERROR == NandEraseBlock(Block))
			{
				sprintf(s,"NAND Block %u Erase Failed Or",Block);
				UartWrite((unsigned char *)s,strlen(s));
				sprintf(s,"NAND is Write Protected\r\n");
				UartWrite((unsigned char *)s,strlen(s));
				tries++;
			}
			else
			{
				Boolean Error = FALSE;

				for(Int32U Page = 0; NAND_PG_PER_BLK > Page; Page++)
				{
					for(Int32U i = 0; (NAND_MAIN_SIZE+4) > i; i++)
					{
						pBuffer[i] = ((Block^Page) + i) & 0xFF;
	
						//Valid Block Mark
						if(((3 == Page) || (7 == Page)) && 464 == i )
							pBuffer[i] = 0xFF;
					}

					if(FLASH_ERROR == NandWritePage(pBuffer, Block*NAND_PG_PER_BLK + Page,0))
					{
						sprintf(s,"NAND Program Error Page: %u Block: %u\r\n", Page, Block);
						UartWrite((unsigned char *)s,strlen(s));
						Error= TRUE;
						tries++;
						break;
					}
				}

				if((FALSE) == Error)
				{	
					//Read whole Block
					for(Int32U Page = 0; NAND_PG_PER_BLK > Page; Page++)
					{
						if(FLASH_ERROR == NandReadPage(pBuffer, Block*NAND_PG_PER_BLK + Page,0))
						{
							sprintf(s,"NAND Read Error Page %u Block %u\r\n", Page, Block);
							UartWrite((unsigned char *)s,strlen(s));
							
							Error = TRUE;
							tries++;
							break;
						}

						for(Int32U i = 0; (NAND_MAIN_SIZE+4) > i; i++)
						{
							unsigned char Test = ((Block^Page) + i) & 0xFF;

							if(((3 == Page) || (7 == Page)) && 464 == i )  Test = 0xFF;

							if( pBuffer[i] != Test)
							{
								sprintf(s,"NAND Read Error Page %u Block %u\r\n", Page, Block);
								UartWrite((unsigned char *)s,strlen(s));
								Error = TRUE;
								tries++;
								break;
							}
						}
					}

				}

				if((FALSE) == Error)
				{
					sprintf(s,"NAND Block: %u O.K.\r\n", Block);
					UartWrite((unsigned char *)s,strlen(s));
				}

			}

		}
	}
	
	sprintf(s,"\n\rNumber of errors: %d\n\r", tries);
	UartWrite((unsigned char *)s,strlen(s));
	sprintf(s,"(test is terminated after 10 errors)\n\r");
	UartWrite((unsigned char *)s,strlen(s));
	
	sprintf(s,"\n\rNAND Test end\n\r");
	UartWrite((unsigned char *)s,strlen(s));

	return tries;
}



